/*
 * Sequence.java
 *
 * Created on 30 de julio de 2002, 18:17
 */

package pgclibrary.objects;
import pgclibrary.dataaccess.*;
import pgclibrary.utilities.*;
import java.sql.*;
import java.util.*;

/**
 *
 * @author  nestor
 */
public class Sequence extends DBObject {
    private long last_value = 0;
    private long increment_by = 0;
    private long max_value = 0;
    private long min_value = 0;
    private long cache_value = 0;
    private boolean cycled = false;
    private boolean called = false;
    private long start = 0;
    
    /** Creates a new instance of Sequence */
    public Sequence(DBObject owner, long oid) {
        super(owner);
        search(oid);
    }
    public Sequence(DBObject owner) {
        super(owner);
    }
    
    /** Retrieve the SQL string that eliminate this object from
     * the database.
     * @return SQL DML string.
     */
    public String getDeleteSQL() {
        StringBuffer retorno = new StringBuffer();
        retorno.append("DROP SEQUENCE ");
        retorno.append(getName()).append(";\n");
        return retorno.toString();
    }
    
    /** Returns the SQL string that
     * generate this object.
     * @return SQL DML instruction for generation.
     */
    public String getInsertSQL() {
        StringBuffer retorno = new StringBuffer();
        retorno.append("CREATE SEQUENCE ");
        retorno.append(getName()).append(" ");
        
        if(getIncrementBy() > 1){
            retorno.append("\n\tINCREMENT ").append(getIncrementBy()).append(" ");
        }
        if(getMinValue() > 0){
            retorno.append("\n\tMINVALUE ").append(getMinValue()).append(" ");
        }
        if(getMaxValue() > 0){
            retorno.append("\n\tMAXVALUE ").append(getMaxValue()).append(" ");
        }
        if(getStart() > 0){
            retorno.append("\n\tSTART ").append(getStart()).append(" ");
        }
        if(getCacheValue() > 0){
            retorno.append("\n\tCACHE ").append(getCacheValue()).append(" ");
        }
        if(isCycled()){
            retorno.append("\n\tCYCLE ");
        }
        retorno.append(";\n");
        return retorno.toString();
    }
    
    /** Retrieve the SQL String that implements this
     * object modifications.
     * @return SQL DML String.
     */
    public String getUpdateSQL() {
        return new String();
    }
    
    /** Find the object asociated with the OID.
     * @param _oid OID to search.
     * @return true if the object was found
     */
    public boolean search(long _oid) {
        boolean retorno = false;
        try{
            initialize();
            PreparedStatement prep = getConnection().prepareStatement("SELECT OID, * FROM PG_CLASS WHERE OID=?");
            prep.setLong(1, _oid);
            ResultSet rs = prep.executeQuery();
            if(rs.next()){
                setName(rs.getString("RELNAME"));
                setOID(rs.getLong("OID"));
                setCreator(new User(this, rs.getLong("RELOWNER")));
                
                PreparedStatement prep1 = getConnection().prepareStatement("SELECT * FROM " + getName());
                ResultSet rs1 = prep1.executeQuery();
                if(rs1.next()){
                    setLastValue(rs1.getLong("LAST_VALUE"));
                    setIncrementBy(rs1.getLong("INCREMENT_BY"));
                    setMaxValue(rs1.getLong("MAX_VALUE"));
                    setMinValue(rs1.getLong("MIN_VALUE"));
                    setCacheValue(rs1.getLong("CACHE_VALUE"));
                    setCycled(rs1.getBoolean("IS_CYCLED"));
                    setCalled(rs1.getBoolean("IS_CALLED"));
                    retorno = true;
                }
                rs1.close();
                prep1.close();
            }
            
            rs.close();
            prep.close();
        } catch (SQLException e){
            initialize();
            e.printStackTrace();
        }
        return retorno;
    }
    
    public void initialize(){
        super.initialize();
        start = 0;
        last_value = 0;
        increment_by = 0;
        max_value = 0;
        min_value = 0;
        cache_value = 0;
        cycled = false;
        called = false;
    }
    
    // ------ SET AND GET --------
    public long getLastValue(){
        return last_value;
    }
    private void setLastValue(long _last_value){
        last_value = _last_value;
    }
    public long getIncrementBy(){
        return increment_by;
    }
    public void setIncrementBy(long _increment_by){
        increment_by = _increment_by;
    }
    
    public long getMaxValue(){
        return max_value;
    }
    public void setMaxValue(long _max_value){
        max_value = _max_value;
    }
    
    public long getMinValue(){
        return min_value;
    }
    public void setMinValue(long _min_value){
        min_value = _min_value;
    }
    
    public long getCacheValue(){
        return cache_value;
    }
    public void setCacheValue(long _cache_value){
        cache_value = _cache_value;
    }
    
    public boolean isCycled(){
        return cycled;
    }
    public void setCycled(boolean _cycled){
        cycled = _cycled;
    }
    
    public boolean isCalled(){
        return called;
    }
    private void setCalled(boolean _called){
        called = _called;
    }
    
    private long getStart(){
        return start;
    }
    public void setStart(long _start){
        start = _start;
    }
}
